module (arg and arg[1])

local function import_config (req, res, params)
    if req.cmd_mth ~= "POST" then
        return httpd.err_405 (req, res)
    end

    local _,_,boundary = string.find (req.headers["content-type"], "boundary%=(.-)$")
    local len = 0
    local content_len_remaining = req.headers["content-length"] + 0
    local MAX_READ_SIZE = 25000
    local read_size
    local data
    local err

    -- read lines until the header is done
    repeat
        data, err = req.socket:receive ()
        if not data then
            trace(0, 1, string.format("import no data"))
            return
        end
        len = string.len(data) + 2
        content_len_remaining = content_len_remaining - len
    until data == "" or err

    if err then
        trace(0, 1, string.format("error on import: %s", err))
        return
    end

    repeat
        if content_len_remaining < MAX_READ_SIZE then
            read_size = content_len_remaining
        else
            read_size = MAX_READ_SIZE
        end

        data, err = req.socket:receive (read_size)
        if err then
            trace(0, 1, string.format("socket:receive error %s", err))
            break
        end

        if not data then
            err = "ran out of data"
            break
        end

        err = msg_if.lua_put_config(data,string.len(data))
        len = len + string.len(data)
        content_len_remaining = content_len_remaining - read_size
    until content_len_remaining == 0 or err
    trace(0, 1, string.format("uploaded %d bytes", len))

    res:send_data("<html>")
    res:send_data("<head>")
    res:send_data('<meta http-equiv="refresh" content="1; url=index.lua?pageID=Administration">')
    res:send_data("</head>")
    if not err then
        res:send_data("<body>Done</body>")
    else 
        trace(0, 1, string.format("error on import: %s", err))
        res:send_data("<body>Import failed</body>")
    end
    res:send_data("</html>")

end


function makeHandler (params)
    return function (req, res)
        return import_config (req, res, params)
    end
end
