-------------------------------------------------------------------------------
-- Xavante configuration file.
--
-- The configuration defines the Xavante environment
-- Commented fields are optional and assume default values
--
-- Xavante defines virtualhosts for each site running with Xavante.
-- Each virtualhost can define the handlers for specific files extensions.
-- Xavante currently offers a fileHandler and a CGILuaHandler.
--
-- Xavante configuration can be redefined on the default file structure by the
-- optional /conf/xavante/config.lua file. If it does not exist, Xavante loads
-- the default /bin/xavante/config.lua file.
--
-- Authors: Javier Guerra and Andre Carregal
-- Copyright (c) 2004-2005 Kepler Project
-------------------------------------------------------------------------------
require "xavante.filehandler"
require "xavante.cgiluahandler"
require "xavante.redirecthandler"
require "debug_info"
require "upload_image"
require "reboot_terminal"
require "export_config"
require "import_config"
require "export_call_log"

local simplerules = {
    {match = "/upload_image", with = upload_image, params = nil },
    {match = "/reboot_terminal", with = reboot_terminal, params = nil },
    {match = "/import_config", with = import_config, params = nil },
    {match = "/config.dat", with = export_config, parms = nil },
    {match = "/call_log.txt", with = export_call_log, params = nil }, 
    {match = "/debug_info", with = debug_info, params = {"date", "ver", "sysconf", "errorlog", 
             "eventhandler -l", "post", "stat -l full", 
             "iai2_cn0_show", "call_log all", "trace -m umts-reg -l 1000 -f g", "trace -m umts-mm -l 1000 -f g",
             "trace -m umts-gmm -l 1000 -f g", "trace -m umts-sm: -l 1000 -f g", "trace -m umts-cc -l 1000 -f g",
             "trace -m umts -l 1000 -f g", "trace -m iai2:bct -l 1000 -f g", "trace -m iai2:bcn -l 1000 -f g",
             "trace -m iai2:rforward -l 1000 -f g", "trace -m iai2:rreturn -l 1000 -f g", 
             "trace -m al:rbc -l 1000 -f g", "trace -m al: -l 1000 -f g", "trace -m isdn -l 1000 -f g",
             "trace -m csup -l 1000 -f g", "trace -m gps -l 1000 -f g", "trace -m pabx -l 1000 -f g",
             "trace -m acm_if -l 200", "trace -m atb_if -l 200", "trace -m search_dsp_ctrl -l 200",
             "trace -m sat_search -l 200", "trace -m al:dspa", "trace -m al:diagnostic -l 1000 -f g",
             "trace -l 1000"} }, 
    {match = "/", with = xavante.redirecthandler, params = {"index.lua"}}, 
    {match = "/*", with = xavante.filehandler, params = {baseDir = xavante.webdir()}},
    {match = {"/*.lp", "/*.lua"},  with = xavante.cgiluahandler.makeHandler (xavante.webdir())},
}

xavante.HTTP{
    server = {host = "*", port = XAVANTE_HTTP_PORT},
    
    defaultHost = {
        rules = simplerules
    },
}
