-- ***************
-- local functions
-- ***************

msn_types = {
   "msn_speech",
   "msn_31khz",
   "msn_UDI",
   "msn_RDI"
}

-- needs verification if this generates correct HTML code
selected_x_text = function (expr)
  if (expr) then
    return ' selected="selected" '
  else
    return ' '
  end
end
   
-- ********************************
-- ISDN obejct handling (set)
-- ********************************

-- Normal / Dual mode is not available yet.
-- if cgilua.cgi["radio_mode_isdn"] ~= nil then
--    if cgilua.cgi["radio_mode_isdn"] == "enabled" then
--       msg_if.lua_set_isdn_dual("normal")
--    elseif cgilua.cgi["radio_mode_isdn"] == "disabled" then
--       msg_if.lua_set_isdn_dual("dual")
--    end
-- end


-- Handle parameters for a given isdn interface.
-- Interfaces are enumerated starting from 0.

-- All parameters specific for a given interface, are 
-- postfixed with the interface number.

function set_msn_number(i, interface, type, type_index)
   put(    '    set_msn_number(' .. i .. ', ' .. interface .. ', ' .. type .. ', ' .. type_index .. ')\n')
   if cgilua.cgi[type .. i] ~= nil then
      m = cgilua.cgi[type .. i]
      put( '      msg_if.lua_set_isdn_msn_number(' .. interface .. ', ' .. type_index .. ', ' .. m .. ')\n')
      msg_if.lua_set_isdn_msn_number(interface, type_index, m)
   end
end

function set_interface_parameters(i, interface)

   put('\n<!-- Set interface parameters for interface: ' .. interface .. ' Index: ' .. i .. '\n')
   calltype, s1, s2, s3, s4 = msg_if.lua_get_isdn_status(interface)
   trace(0, 1, string.format("Interface lua(%d) - (s1, s2, s3, s4) == (%d, %d, %d, %d)", 
                 i, s1, s2, s3, s4))

   if cgilua.cgi["submit"] ~= nil and cgilua.cgi["calltype" .. i] ~= nil then
   
      -- outgoing enable settings
      calltype = cgilua.cgi["calltype" .. i]      
      
      -- incoming routing settings
      if cgilua.cgi["isdnincoming_std" .. i] ~= nil then
     s1 = 1
      else
     s1 = 0
      end
      
      if cgilua.cgi["isdnincoming_prem" .. i] ~= nil then
     s2 = 1
      else
     s2 = 0
      end
      
      if cgilua.cgi["isdnincoming_udi" .. i] ~= nil then
     s3 = 1
      else
     s3 = 0
      end
      
      if cgilua.cgi["isdnincoming_rdi" .. i] ~= nil then
     s4 = 1
      else
     s4 = 0
      end
      
      msg_if.lua_set_isdn_status(interface,calltype,s1,s2,s3,s4)
      put('  msg_if.lua_set_isdn_status(' .. interface .. ', ' .. calltype .. ', ' 
      .. s1 .. ', ' .. s2 .. ', ' .. s3 .. ', ' .. s4 .. ')\n')
      for type_index, type in ipairs(msn_types) do
     set_msn_number(i, interface, type, type_index-1)
      end
      put('Done.')
   else
      put('No settings set')
   end
   put(' -->\n')
end



-- ****************************************
-- * Generate HTML for a MSN dropdown box *
-- * HTML is send to standard out         *
-- ****************************************
function generate_msn_dropdown_html(interface, type_index, name)
   put('<!-- get msn: ' .. interface .. ', ' .. type_index .. '-->\n');
   selected, entries = msg_if.lua_get_isdn_msn_numbers(interface, type_index - 1);
   put('<select ' .. disabled_string .. '  name="' .. name .. '">\n')
   a = {}
   for i in pairs(entries) do table.insert(a, i) end
   table.sort(a)
   for i, n in ipairs(a) do
      put('  <OPTION' .. selected_x_text(n == selected) .. 'value="' .. entries[n] .. '">')
      str = entries[n]
      if str == '' then
     str = '<' .. current_dictionary("None") .. '>'
      end
      put(html_entities(str))
      put('</OPTION>\n')
   end
   put('</SELECT>\n')   
end
   

-- ****************************************
-- * Generate HTML for a single interface *
-- * The HTML is printed to std-out       *
-- ****************************************
function generate_interface_html(i, interface, port)
   calltype, s1, s2, s3, s4 = msg_if.lua_get_isdn_status(interface)

   put('<tr>\n')
   put('  <th align="left"><p class="menutext">' .. current_dictionary("port") .. ' ' .. port .. '</p></th>\n')
   put('</tr>\n') 

   -- Incoming calls
   put('<tr>\n')

   put('  <td align="left"><p>' ..current_dictionary("incoming calls").. '</p>\n')
   put('  </td>\n')

   put('  <td align="left"><p>\n')
   put('    <input ' .. disabled_string .. ' type="checkbox" name="isdnincoming_std' .. i .. '"\n')
   put('      value="ON"' .. checked_text(s1 == 1) .. '>' ..current_dictionary("standard").. '</p>\n')
   put('  </td>\n')

   put('  <td align="left"><p>\n')
   put('    <input ' .. disabled_string .. ' type="checkbox" name="isdnincoming_prem' .. i .. '"\n')
   put('      value="ON"' .. checked_text(s2 == 1) .. '>' ..current_dictionary("3.1kHz audio").. '</p>\n')
   put('  </td>\n')

   put('  <td align="left"><p>\n')
   put('    <input ' .. disabled_string .. ' type="checkbox" name="isdnincoming_udi' .. i .. '"\n')
   put('      value="ON"' .. checked_text(s3 == 1) .. '>UDI</p>\n')
   put('  </td>\n')

   put('  <td align="left"><p>\n')
   put('    <input ' .. disabled_string .. ' type="checkbox" name="isdnincoming_rdi' .. i .. '"\n')
   put('      value="ON"' .. checked_text(s4 == 1) .. '>RDI</p>\n')
   put('  </td>\n')
   
   -- MSN Numbers
   put('<tr>\n')

   put('  <td align="left"><p>' ..current_dictionary("MSN numbers").. '</p>\n')
   put('  </td>\n')

   -- generate the drop down box. 
   --   m1 - m4 are the selcted values for the interface
   for type_index = 1,4 do
      put('  <td align="left"><p>\n')
      generate_msn_dropdown_html(interface, type_index, msn_types[type_index] .. i)
      put('  </td>\n')
   end

   -- Outgoing calls
   put('<tr>\n')
   put('  <td align="left"><p>' ..current_dictionary("outgoing calls") .. '</p>\n')
   put('  </td>\n')
   put('  <td align="left"><p>\n')
   put('    <input ' .. disabled_string .. ' type="radio" name="calltype' .. i .. '"\n')
   put('           value="4" ' ..checked_text(calltype == 4).. '>' .. current_dictionary("isdn cs out automatic"))
   put('    </p>\n')
   put('  </td>\n')
   put('  <td align="left"><p>\n')
   put('    <input ' .. disabled_string .. ' type="radio" name="calltype' .. i .. '"\n')
   put('           value="0" ' ..checked_text(calltype == 0).. '>' .. current_dictionary("standard"))
   put('    </p>\n')
   put('  </td>\n')
   put('  <td align="left"><p>\n')
   put('    <input ' .. disabled_string .. ' type="radio" name="calltype' .. i .. '"\n')
   put('           value="1" ' ..checked_text(calltype == 1).. '>' .. current_dictionary("3.1kHz audio"))
   put('    </p>\n')
   put('  </td>\n')
   put('</tr>\n')
end


-- *****************
-- get current state
-- *****************

-- Get list of isdn interfaces id's
interface_table = msg_if.lua_get_isdn_interfaces()

-- Store all interface parameters if nessesary.
for i,interface in ipairs(interface_table) do
   set_interface_parameters(i, interface)
end

if cgilua.cgi["submit"] ~= nil then
   if cgilua.cgi["radio_onoff_isdn"] ~= nil then
      if cgilua.cgi["radio_power_isdn"] == "enabled" then
     power = 1
      else
     power = 0
      end
      msg_if.lua_set_isdn_power_status(power);
   end
end

status = msg_if.lua_read_interface_status("isdn")
power = msg_if.lua_get_isdn_power_status()
--mode = msg_if.lua_isdn_dual_status()

-- ***********
-- format html
-- ***********

-- check access permission
    disabled_string = ''
    if admin_status < 0 then
        allowed = msg_if.lua_get_user_permissions("change_isdn")
        if not allowed then 
            disabled_string = ' disabled '
            error_msg = current_dictionary("admin_locked")
        end
    end



put(start_of_content)
put('<p id="errortext">' .. error_msg .. '</p>')
put('<form name="settings_interfaces" method="post" action="index.lua?pageID=' .. page_title)
put('&langID=' .. lang .. '">\n')
put('<p><table border="'..border_on..'" cellpadding="3" align="left" valign="top">\n')


if product == "EBDU" or product == "E727" or product == "E723" or product == "SBDU" or product == "S500" or product == "S250" or product == "SBL" then
else
    --Enable/disable interface
    put('<tr>')
    put('<th align="left" colspan="4"><p class="menutext">' ..current_dictionary("interface status"))
    put('</p></th>\n')
    put('<th align="left" ><p class="menutext"></p></th>\n')
    put('<th align="left" ><p class="menutext"></p></th>\n')
    put('<th align="left" ><p class="menutext"></p></th>\n')
    put('</tr>')

    -- Enable / disable
    put('<tr>')
    put('<td align="left"><p>' ..current_dictionary("isdn interface") .. '</p></td>\n')
    put('<td align="left"><p><input ' .. disabled_string .. ' type="radio" name="radio_onoff_isdn" ')
    put('value="enabled" ' ..checked_text(status == 0).. '>' .. current_dictionary("enabled"))
    put('</p></td>\n')
    put('<td align="left"><p><input ' .. disabled_string .. ' type="radio" name="radio_onoff_isdn" ')
    put('value="disabled" ' ..checked_text(status == 1).. '>' .. current_dictionary("disabled"))
    put('</p></td>\n')
    put('</tr>')
    -- Power 38V
    put('<tr>')
    put('<td align="left"><p>' ..current_dictionary("power(38V) interface") .. '</p></td>\n')
    put('<td align="left"><p><input ' .. disabled_string .. ' type="radio" name="radio_power_isdn" ')
    put('value="enabled" ' ..checked_text(power ~= 0).. '>' .. current_dictionary("enabled"))
    put('</p></td>\n')
    put('<td align="left"><p><input ' .. disabled_string .. ' type="radio" name="radio_power_isdn" ')
    put('value="disabled" ' ..checked_text(power == 0).. '>' .. current_dictionary("disabled"))
    put('</p></td>\n')
    put('</tr>')
end

--Dual mode not working in 1.04, so the option is disabled for now
--put('<tr>')
--    put('<td align="left"><p>' ..current_dictionary("operation mode") .. '</p></td>')
--    put('<td align="left"><p><input ' .. disabled_string .. ' type="radio" name="radio_mode_isdn" value="enabled" ' ..checked_text(mode == 0).. '>' .. current_dictionary("normal") .. '</p></td>\n')
--    put('<td align="left"><p><input ' .. disabled_string .. ' type="radio" name="radio_mode_isdn" value="disabled" ' ..checked_text(mode == 1).. '>' .. current_dictionary("dual") .. '</p></td>\n')
--put('</tr>')

-- Get list of isdn interfaces id's. Re-read the list, as the interface might just have been enabled.

if table.getn(interface_table) == 0 then
   put('\n<!-- sleep -->')
   msg_if.lua_sleep(1000)
   interface_table = msg_if.lua_get_isdn_interfaces()
end


a = {}
rev_interface = {}
for i, j in ipairs(interface_table) do 
   table.insert(a, j) 
   rev_interface[j] = i
end
table.sort(a)
for i, n in ipairs(a) do
   put('<!-- i:' .. i .. ' n:' .. n .. ' rev_interface: ' .. rev_interface[n] .. '-->')
   generate_interface_html(rev_interface[n], n, i)
end

--submit button
put('  <tr>\n')
put('    <td align="left"><p>\n')
put('      <input ' .. disabled_string .. ' type="submit" name="submit" value="' .. current_dictionary("apply") .. '"/>\n')
put('        &nbsp;&nbsp;\n')
put('      <input ' .. disabled_string .. ' type="reset" value="' .. current_dictionary("cancel") .. '"/></p>\n')
put('    </td>\n')
put('  </tr>\n')
put('</table></form></p>\n')
put(end_of_content)

