-- Glbal variable definitions and initializations
max_filter_id = 19     -- 20 filters with id 0..19

eval_prec_ix = {}
ip_addr_1 = {}
ip_addr_2 = {}
ip_addr_3 = {}
ip_addr_4 = {}
sub_mask_1 = {}
sub_mask_2 = {}
sub_mask_3 = {}
sub_mask_4 = {}
prot_no = {}
dest_port_range_from = {}
dest_port_range_to = {}
source_port_range_from = {}
source_port_range_to = {}
type_of_service = {}
type_of_service_mask = {}

used_epis = {}
sorted_epi_list = {}

blank_num = 4294967295 -- 0xffffffff

-- Function declarations

store_parm = function(cgi, parm)
    if cgi ~= nil then                     
        if cgi == "" then
            return blank_num
        else 
            return cgi
        end    
    else
        return parm
    end    
end

get_input = function(index)
    eval_prec_ix[index] = store_parm(cgilua.cgi['EPI_INPUT'], eval_prec_ix[index])
    ip_addr_1[index] = store_parm(cgilua.cgi['ADRESS_INPUT_1'], ip_addr_1[index])
    ip_addr_2[index] = store_parm(cgilua.cgi['ADRESS_INPUT_2'], ip_addr_2[index])
    ip_addr_3[index] = store_parm(cgilua.cgi['ADRESS_INPUT_3'], ip_addr_3[index])
    ip_addr_4[index] = store_parm(cgilua.cgi['ADRESS_INPUT_4'], ip_addr_4[index])

    sub_mask_1[index] = store_parm(cgilua.cgi['SUBNET_INPUT_1'], sub_mask_1[index])
    sub_mask_2[index] = store_parm(cgilua.cgi['SUBNET_INPUT_2'], sub_mask_2[index])
    sub_mask_3[index] = store_parm(cgilua.cgi['SUBNET_INPUT_3'], sub_mask_3[index])
    sub_mask_4[index] = store_parm(cgilua.cgi['SUBNET_INPUT_4'], sub_mask_4[index])
       
    prot_no[index] = store_parm(cgilua.cgi['PROT_NO_INPUT'], prot_no[index])
        
    dest_port_range_from[index] = store_parm(cgilua.cgi['DEST_PORT_FROM_INPUT'], dest_port_range_from[index])
    dest_port_range_to[index] = store_parm(cgilua.cgi['DEST_PORT_TO_INPUT'], dest_port_range_to[index])

    source_port_range_from[index] = store_parm(cgilua.cgi['SOURCE_PORT_FROM_INPUT'], source_port_range_from[index])
    source_port_range_to[index] = store_parm(cgilua.cgi['SOURCE_PORT_TO_INPUT'], source_port_range_to[index])

    type_of_service[index] = store_parm(cgilua.cgi['SERVICE_TYPE'], type_of_service[index])
    type_of_service_mask[index] = store_parm(cgilua.cgi['SERVICE_TYPE_MASK'], type_of_service_mask[index])
end

check_unique_eval_index = function()
    for i = 0, max_filter_id do
        -- Check for duplicate values of eval prec index
        if i > 0 then
            epi = eval_prec_ix[i]
            if epi ~= blank_num then
                for j = 0, i - 1 do
                    if eval_prec_ix[j] == epi then
                        return 100, i --Error return
                    end
                end
            end
        end
    end
   return 0, 0
end

sort_by_epi = function()
    sorted_epi_list = { -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}
    sorted_epi_list[0] = -1
    for i = 0, max_filter_id do
        epi = eval_prec_ix[i]
        if epi ~= blank_num then
            if epi > -1 and epi < 20 then
                trace(0, 1, string.format("found epi %d at index %d",epi, i))
                sorted_epi_list[epi] = i;
            end
        end
    end
end

set_filter = function(index)
    err_val = tft_if.set_tft_filter(eval_prec_ix[index], ip_addr_1[index], ip_addr_2[index], ip_addr_3[index], ip_addr_4[index], sub_mask_1[index], sub_mask_2[index], sub_mask_3[index], sub_mask_4[index], prot_no[index], dest_port_range_from[index], dest_port_range_to[index], source_port_range_from[index], source_port_range_to[index], type_of_service[index], type_of_service_mask[index], index)
    return err_val
end

get_filters = function()
    first_free_filter_index = -1;
    used_epis = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }
    used_epis[0] = 0

    for i = 0, max_filter_id do
        eval_prec_ix[i], ip_addr_1[i], ip_addr_2[i], ip_addr_3[i], ip_addr_4[i], sub_mask_1[i], sub_mask_2[i], sub_mask_3[i],
        sub_mask_4[i], prot_no[i], dest_port_range_from[i], dest_port_range_to[i], source_port_range_from[i], source_port_range_to[i], 
        type_of_service[i], type_of_service_mask[i] = tft_if.get_tft_filter(i)

        if (eval_prec_ix[i] == 4294967295) then
            if (first_free_filter_index == -1) then
                first_free_filter_index = i
            end
        else 
            used_epis[eval_prec_ix[i]] = 1
        end
    end
    trace(0, 1, string.format("used_epis %d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d",used_epis[0], used_epis[1], used_epis[2], used_epis[3], used_epis[4], used_epis[5], used_epis[6], used_epis[7], used_epis[8], used_epis[9], used_epis[10], used_epis[11], used_epis[12], used_epis[13], used_epis[14], used_epis[15], used_epis[16], used_epis[17], used_epis[18], used_epis[19]))
end

filter_profiles = {}

-- clone a filters and assign to new profiles
-- only used when migrating from an old version that supports
-- multiple profiles per filter.
-- Also, we need to make sure that cloned filters are ordered
migrate_filters = function()

    temp_eval_prec_ix = {}
    temp_ip_addr_1 = {}
    temp_ip_addr_2 = {}
    temp_ip_addr_3 = {}
    temp_ip_addr_4 = {}
    temp_sub_mask_1 = {}
    temp_sub_mask_2 = {}
    temp_sub_mask_3 = {}
    temp_sub_mask_4 = {}
    temp_prot_no = {}
    temp_dest_port_range_from = {}
    temp_dest_port_range_to = {}
    temp_source_port_range_from = {}
    temp_source_port_range_to = {}
    temp_type_of_service = {}
    temp_type_of_service_mask = {}
    temp_filter_profiles = {}

    -- copy filters to temp lists
    --number_of_filters = 0;

    for i = 0, max_filter_id do
        if eval_prec_ix[i] >= 0 then 
            temp_eval_prec_ix[i] = eval_prec_ix[i] * 10;
            temp_ip_addr_1[i] = ip_addr_1[i]
            temp_ip_addr_2[i] = ip_addr_2[i]
            temp_ip_addr_3[i] = ip_addr_3[i]
            temp_ip_addr_4[i] = ip_addr_4[i]
            temp_sub_mask_1[i] = sub_mask_1[i]
            temp_sub_mask_2[i] = sub_mask_2[i]
            temp_sub_mask_3[i] = sub_mask_3[i]
            temp_sub_mask_4[i] = sub_mask_4[i]
            temp_prot_no[i] = prot_no[i]
            temp_dest_port_range_from[i] = dest_port_range_from[i]
            temp_dest_port_range_to[i] = dest_port_range_to[i]
            temp_source_port_range_from[i] = source_port_range_from[i]
            temp_source_port_range_to[i] = source_port_range_to[i]
            temp_type_of_service[i] = type_of_service[i]
            temp_type_of_service_mask[i] = type_of_service_mask[i]
            temp_filter_profiles[i] = filter_profiles[i]
            --number_of_filters = number_of_filters + 1;
        end
    end

    rows, cols, checked = tft_if.get_tft() 
    running_index = 1;
    total_clones = 0;

    -- if same filter is set on str256 and u1/u2 precendence will be swapped (str256 getting higher prec val)
    for i=0, rows-1 do
        found = 0;
        number_of_clones = 0;
        temp_filter_profiles[i] = -1;
        for j=0, cols-1 do
             --there might be more than one profile pernumber_of_filters + total_clones filter.
             if string.byte(checked, running_index) == string.byte('1') then
                 if found == 0 then
                     temp_filter_profiles[i] = j;
                     found = 1;
                     trace(0, 1, string.format("filter: %d, profile: %d", i, temp_filter_profiles[i]))
                 else 
                    --we already found a profile for this filter. since the new gui only supports one profile per filter, we need 
                    --to clone the filter so each profile can be set on a different filter!

                    trace(0, 1, string.format("cloning filter, %d", i))

                    number_of_clones = number_of_clones + 1
                    total_clones = total_clones + 1

                    -- create new filter. and add it to the end of list. It does not matter if length of list exceeds 20
                    -- as we will take the 20 lowest eval. indexes. and use as the final list 

                    temp_eval_prec_ix[max_filter_id+total_clones] = eval_prec_ix[i] * 10 + number_of_clones;
                    temp_ip_addr_1[max_filter_id+total_clones] = ip_addr_1[i]
                    temp_ip_addr_2[max_filter_id+total_clones] = ip_addr_2[i]
                    temp_ip_addr_3[max_filter_id+total_clones] = ip_addr_3[i]
                    temp_ip_addr_4[max_filter_id+total_clones] = ip_addr_4[i]
                    temp_sub_mask_1[max_filter_id+total_clones] = sub_mask_1[i]
                    temp_sub_mask_2[max_filter_id+total_clones] = sub_mask_2[i]
                    temp_sub_mask_3[max_filter_id+total_clones] = sub_mask_3[i]
                    temp_sub_mask_4[max_filter_id+total_clones] = sub_mask_4[i]
                    temp_prot_no[max_filter_id+total_clones] = prot_no[i]
                    temp_dest_port_range_from[max_filter_id+total_clones] = dest_port_range_from[i]
                    temp_dest_port_range_to[max_filter_id+total_clones] = dest_port_range_to[i]
                    temp_source_port_range_from[max_filter_id+total_clones] = source_port_range_from[i]
                    temp_source_port_range_to[max_filter_id+total_clones] = source_port_range_to[i]
                    temp_type_of_service[max_filter_id+total_clones] = type_of_service[i]
                    temp_type_of_service_mask[max_filter_id+total_clones] = type_of_service_mask[i]
                    temp_filter_profiles[max_filter_id + total_clones] = j
                    --filter_profiles[max_filter_id+total_clones] = j;
                    trace(0, 1, string.format("filter: %d, profile: %d ( clone )", max_filter_id+total_clones, temp_filter_profiles[max_filter_id+total_clones]))
                 end
             end
             running_index = running_index + 1;
        end
    end

    temp_epi_map = {}

    -- map epi's to indexes for sorting
    for i=0, max_filter_id + total_clones do
         if temp_eval_prec_ix[i] ~= nil then
             epi = temp_eval_prec_ix[i]
             temp_epi_map[epi] = i;
         end
    end

    -- normalise epi's and insert first 20 into list
    epi_counter = 0;
    for i=0, 210 do
        if ( temp_epi_map[i] ~= nil ) and (temp_filter_profiles[temp_epi_map[i]] ~= -1) and ( epi_counter < 20 ) then
            trace (0, 1, string.format("adding filter with index: %d, with epi: %d and profile %d", temp_epi_map[i], epi_counter + 1, temp_filter_profiles[temp_epi_map[i]]))
            eval_prec_ix[epi_counter] = epi_counter +1
            ip_addr_1[epi_counter] = temp_ip_addr_1[temp_epi_map[i]]
            ip_addr_2[epi_counter] = temp_ip_addr_2[temp_epi_map[i]]
            ip_addr_3[epi_counter] = temp_ip_addr_3[temp_epi_map[i]]
            ip_addr_4[epi_counter] = temp_ip_addr_4[temp_epi_map[i]]
            sub_mask_1[epi_counter] = temp_sub_mask_1[temp_epi_map[i]]
            sub_mask_2[epi_counter] = temp_sub_mask_2[temp_epi_map[i]]
            sub_mask_3[epi_counter] = temp_sub_mask_3[temp_epi_map[i]]
            sub_mask_4[epi_counter] = temp_sub_mask_4[temp_epi_map[i]]
            prot_no[epi_counter] = temp_prot_no[temp_epi_map[i]]
            dest_port_range_from[epi_counter] = temp_dest_port_range_from[temp_epi_map[i]]
            dest_port_range_to[epi_counter] = temp_dest_port_range_to[temp_epi_map[i]]
            source_port_range_from[epi_counter] = temp_source_port_range_from[temp_epi_map[i]]
            source_port_range_to[epi_counter] = temp_source_port_range_to[temp_epi_map[i]]
            type_of_service[epi_counter] = temp_type_of_service[temp_epi_map[i]]
            type_of_service_mask[epi_counter] = temp_type_of_service_mask[temp_epi_map[i]]
            filter_profiles[epi_counter] = temp_filter_profiles[temp_epi_map[i]]
            epi_counter = epi_counter + 1;
        end
    end

    --and for the grand finale, store everything in the flash database!
    for i=0, max_filter_id do
        set_filter( i );
        set_profile_mapping( i, filter_profiles[i] );
    end;
end

-- Get the string encoding the profile for all filters
get_profile_mapping = function()
    rows, cols, checked = tft_if.get_tft() 
    running_index = 1;

    for i=0, rows-1 do
        filter_profiles[i] = -1;
        for j=0, cols-1 do
             --need to be carefull here. If migrating from an old version, there might be more than one profile per filter.
             if string.byte(checked, running_index) == string.byte('1') then
                 filter_profiles[i] = j;
             end
             running_index = running_index + 1;
        end
        trace(0, 1, string.format("%d, %d", i, filter_profiles[i]))
    end
end

-- Get the string encoding the profile for all filters
migration_needed = function()
    rows, cols, checked = tft_if.get_tft() 
    running_index = 1;
    migrate = false;

    for i=0, rows-1 do
        found = 0;
        filter_profiles[i] = -1;
        for j=0, cols-1 do
             --need to be carefull here. If migrating from an old version, there might be more than one profile per filter.
             if string.byte(checked, running_index) == string.byte('1') then
                 if found == 0 then
                     found = 1;
                 else 
                    migrate = true;
                    trace(0, 1, "Migration needed!")
                 end
             end
             running_index = running_index + 1;
        end
    end
    return migrate;
end

-- insert the selected profile into the profile string and save it.
-- a filters profile mapping is of the form 00001000 with the index 
-- corrosponding to the profile set to 1. -1 means that there are no profiles
-- for this filter
set_profile_mapping = function(filter, profile)
    rows, cols, checked = tft_if.get_tft() 
    filter_start_index = 1 + filter*cols

    trace(0, 1, string.format("set_profile_mapping filter: %d,  profile %d",filter, profile))

    filter_string = ''
    for i=1,cols do
        if i == profile + 1 then
           filter_string = filter_string .. '1'
        else
           filter_string = filter_string .. '0'
        end
    end

    checked_new = string.sub(checked,1, filter_start_index - 1) .. filter_string .. string.sub(checked, filter_start_index + cols)
    error_msg = tft_if.set_tft(checked_new)
    return error_msg
end

-- If number == 0xffffffff return '' else return number
number_or_blank = function(number)
    if (number == 4294967295) then
        return ''
    else
        return number 
    end       
end

-- if there is a profile, return its name, otherwie return emptu string
-- this is to avoid trying to do stuff with nill values
profile_or_blank = function(number)
    if (filter_profiles[number] == -1) then
        return ''
    else
        idx = filter_profiles[number]
        if streaming == "STREAMING_32_64" and idx > 2 then
            idx = idx + 1
        elseif streaming == "STREAMING_32_256" then
            if idx == 4 then
                idx = 7
            elseif idx > 4 then
                idx = idx - 1 
            end
        end
        return profiles_if.get_profile_name( idx )
    end       
end


put_filter = function (filter_id)
    put('<tr>\n')

    trace(0, 1, string.format("putting filter %d with epi %d",filter_id, eval_prec_ix[filter_id]))
    put('<td align="center"><p>' ..number_or_blank(eval_prec_ix[filter_id]) .. '</p></td>\n')
    put('<td align="center"><p>' .. profile_or_blank(filter_id) .. '</p></td>\n')

    dot_string = ''
    if ip_addr_1[filter_id] ~= blank_num then 
        dot_string = '.'
    end

    put('<td align="center">\n')
        put('<p>' .. number_or_blank(ip_addr_1[filter_id]).. dot_string )
        put(number_or_blank(ip_addr_2[filter_id]) .. dot_string )
        put(number_or_blank(ip_addr_3[filter_id]) .. dot_string )
        put(number_or_blank(ip_addr_4[filter_id]) .. '</p>\n')
    put('</td>\n')

    dot_string = ''
    if sub_mask_1[filter_id] ~= blank_num then 
        dot_string = '.'
    end

    put('<td align="center">\n')
        put('<p>' .. number_or_blank(sub_mask_1[filter_id]) .. dot_string )
        put(number_or_blank(sub_mask_2[filter_id]) .. dot_string )
        put(number_or_blank(sub_mask_3[filter_id]) .. dot_string )
        put(number_or_blank(sub_mask_4[filter_id]) .. '</p>\n')
    put('</td>\n')
    put('<td align="center"><p>' .. number_or_blank(prot_no[filter_id]) .. '</p></td>\n')

    to_string = ''
    if dest_port_range_to[filter_id] ~= 4294967295 then
        to_string = ' to '
    end     

    put('<td align="center"><p>' .. number_or_blank(dest_port_range_from[filter_id]) .. to_string .. number_or_blank(dest_port_range_to[filter_id]) .. '</p></td>\n')

    to_string = ''
    if source_port_range_to[filter_id] ~= 4294967295 then
        to_string = ' to '
    end     

    put('<td align="center"><p>' .. number_or_blank(source_port_range_from[filter_id]).. to_string ..number_or_blank(source_port_range_to[filter_id])..'</p></td>\n')
    put('<td align="center"><p>' ..number_or_blank(type_of_service[filter_id]) .. '</p></td>\n')
    put('<td align="center"><p>' ..number_or_blank(type_of_service_mask[filter_id]) .. '</p></td>\n')
    put('<td align="center"><p><a href="index.lua?pageID=' .. page_title .. '&amp;langID=' .. lang .. '&amp;edit_filter=' .. filter_id .. '">' ..current_dictionary("edit") .. '</a></p></td>\n')
    put('<td align="center"><p><a href="index.lua?pageID=' .. page_title .. '&amp;langID=' .. lang .. '&amp;delete_filter=' .. filter_id .. '">' ..current_dictionary("delete") .. '</a></p></td>\n')
    put('</tr>\n')
end

show_edit_page = false

if cgilua.cgi["edit_filter"] ~= nil then
    show_edit_page = true
end

error_val = 0
filter_no = 0
get_filters()

if migration_needed() then
    migrate_filters()
end

get_profile_mapping()

if cgilua.cgi["submit_new"] ~= nil then
    if cgilua.cgi["save_filter_index"] ~= nil then
        save_filter_index = tonumber( cgilua.cgi["save_filter_index"] )
        get_input(save_filter_index)
        error_val, filter_no = check_unique_eval_index()
        if error_val == 0 then
            trace(0, 1, string.format("Saving filter: %d",save_filter_index))
            error_val = set_filter(save_filter_index)
            if error_val ~= 0 then
                filter_no = save_filter_index;
            end
            trace(0, 1, string.format("set_filter return code: %d",error_val))
        end

        if cgilua.cgi["PROFILE_INPUT"] ~= nil then
            filter_profile = tonumber( cgilua.cgi["PROFILE_INPUT"] )
            set_profile_mapping (save_filter_index, filter_profile) 
            if error_val ~= 0 then
                filter_no = save_filter_index;
            end
        end
        get_filters()
        get_profile_mapping()
    end

    if error_val ~= 0 then
        show_edit_page = true
        edit_filter = save_filter_index
        get_input(edit_filter)
    end
elseif  cgilua.cgi["delete_filter"] ~= nil then
    delete_filter_index = tonumber( cgilua.cgi["delete_filter"] )

    eval_prec_ix[delete_filter_index] = blank_num
    ip_addr_1[delete_filter_index] = blank_num
    ip_addr_2[delete_filter_index] = blank_num
    ip_addr_3[delete_filter_index] = blank_num
    ip_addr_4[delete_filter_index] = blank_num
    sub_mask_1[delete_filter_index] = blank_num
    sub_mask_2[delete_filter_index] = blank_num
    sub_mask_3[delete_filter_index] = blank_num
    sub_mask_4[delete_filter_index] = blank_num
    prot_no[delete_filter_index] = blank_num
    dest_port_range_from[delete_filter_index] = blank_num
    dest_port_range_to[delete_filter_index] = blank_num
    source_port_range_from[delete_filter_index] = blank_num
    source_port_range_to[delete_filter_index] = blank_num
    type_of_service[delete_filter_index] = blank_num
    type_of_service_mask[delete_filter_index] = blank_num

    error_val = set_filter(delete_filter_index)
    trace(0, 1, string.format("set_filter (delete) return code: %d",error_val))

    error_mes = set_profile_mapping(delete_filter_index, -1)
    trace(0, 1, string.format("set_profile_mapping (delete) return message: %s",error_mes))
end


--Generate html code
put(start_of_content)

-- the edit area

if show_edit_page == true then

    headding_string = current_dictionary("edit filter")
    if cgilua.cgi["new_filter"] ~= nil then
        headding_string = current_dictionary("enter values for new filter")
    end
    
    if edit_filter == nil then
        edit_filter = tonumber( cgilua.cgi["edit_filter"] )
    end

    if error_val ~= 0 then
        put('<table align=center><tr><td><p id="errortext">\n')
        put(current_dictionary("error in filter") .. '</p></td></tr></table>\n')
    end    

    text_std = profiles_if.get_profile_name(0)
    text_32  = profiles_if.get_profile_name(1)
    text_64  = profiles_if.get_profile_name(2)
    text_128 = profiles_if.get_profile_name(3)
    text_1   = profiles_if.get_profile_name(4)
    text_2   = profiles_if.get_profile_name(5)
    text_3   = profiles_if.get_profile_name(6)
    text_256 = profiles_if.get_profile_name(7)

    put('<table id="contents" border="'..border_on..'" align="left" width="70%">\n')
    put('  <tbody>\n')
    put('<form method="post" action="index.lua?pageID=' .. page_title .. '&amp;langID=' .. lang .. '&amp;save_filter_index=' .. edit_filter .. '">')
    put('    <tr> <td colspan="2" align="left"><p class="menutext">' .. headding_string .. ' </p></td>\n')
    put('    </tr>\n')
    put('    <tr>\n')
    put('      <td  align="left"><p>' .. string.gsub(current_dictionary("eval.prec.index"),"!"," ") .. ':</p></td>\n')
    put('      <td align="left" width="160">\n')
    put('        <select name="EPI_INPUT" maxlength="3">')
    for i = 0, max_filter_id do
        if used_epis[i] == 0 then
            put ('          <option value="' .. i .. '"> ' .. i .. '</option>\n')
        elseif i == tonumber( eval_prec_ix[edit_filter] ) then
            put ('          <option value="' .. i .. '" selected="selected" >' .. i .. '</option>\n')
        end
    end
    put('        </select></td>\n')
    put('    </tr>\n')
    put('    <tr>\n')
    put('      <td  align="left"><p>' .. current_dictionary("profile") .. '</p></td>\n')
    put('      <td align="left" width="160">\n')
    trace(0, 1, string.format("edit_filter: %d",edit_filter))
    trace(0, 1, string.format("filter_profiles: %d",filter_profiles[edit_filter]))
    put('        <select name="PROFILE_INPUT">')
    put ('          <option value="0" ' .. selected_text(filter_profiles[edit_filter] == 0) .. ' >' .. text_std .. '</option>\n')
    put ('          <option value="1" ' .. selected_text(filter_profiles[edit_filter] == 1) .. ' >' .. text_32 .. '</option>\n')
    put ('          <option value="2" ' .. selected_text(filter_profiles[edit_filter] == 2) .. ' >' .. text_64 .. '</option>\n')
    
    if streaming == "STREAMING_32_64" then
        put ('          <option value="3" ' .. selected_text(filter_profiles[edit_filter] == 3) .. ' >' .. text_1 .. '</option>\n')
        put ('          <option value="4" ' .. selected_text(filter_profiles[edit_filter] == 4) .. ' >' .. text_2 .. '</option>\n')
        put ('          <option value="5" ' .. selected_text(filter_profiles[edit_filter] == 5) .. ' >' .. text_3 .. '</option>\n')
    elseif streaming == "STREAMING_32_256" then
        put ('          <option value="3" ' .. selected_text(filter_profiles[edit_filter] == 3) .. ' >' .. text_128 .. '</option>\n')
        put ('          <option value="4" ' .. selected_text(filter_profiles[edit_filter] == 4) .. ' >' .. text_256 .. '</option>\n')
        put ('          <option value="5" ' .. selected_text(filter_profiles[edit_filter] == 5) .. ' >' .. text_1 .. '</option>\n')
        put ('          <option value="6" ' .. selected_text(filter_profiles[edit_filter] == 6) .. ' >' .. text_2 .. '</option>\n')
    else
        put ('          <option value="3" ' .. selected_text(filter_profiles[edit_filter] == 3) .. ' >' .. text_128 .. '</option>\n')
        put ('          <option value="4" ' .. selected_text(filter_profiles[edit_filter] == 4) .. ' >' .. text_1 .. '</option>\n')
        put ('          <option value="5" ' .. selected_text(filter_profiles[edit_filter] == 5) .. ' >' .. text_2 .. '</option>\n')
        put ('          <option value="6" ' .. selected_text(filter_profiles[edit_filter] == 6) .. ' >' .. text_3 .. '</option>\n')
    end
    put('        </select></td>\n')
    put('    </tr>\n')
    put('    <tr>\n')
    put('      <td align="left"><p title = "' .. current_dictionary("range for each dot-separated field:") .. '0..255">' ..current_dictionary("source address") .. '</p></td>\n')
    put('      <td align="left" width="160"><input name="ADRESS_INPUT_1" maxlength="3" size="3" value="' .. number_or_blank(ip_addr_1[edit_filter]) .. '" type="text">.<input name="ADRESS_INPUT_2" maxlength="3" size="3" value="' .. number_or_blank(ip_addr_2[edit_filter]) .. '" type="text">.<input name="ADRESS_INPUT_3" maxlength="3" size="3" value="' .. number_or_blank(ip_addr_3[edit_filter]) .. '" type="text">.<input name="ADRESS_INPUT_4" maxlength="3" size="3" value="' .. number_or_blank(ip_addr_4[edit_filter]) .. '" type="text"></td>\n')
    put('    </tr>\n')
    put('    <tr>\n')
    put('      <td align="left"><p title = "' .. current_dictionary("range for each dot-separated field:") .. '0..255" >' .. current_dictionary("subnet mask") .. '</p></td>\n')
    put('      <td align="left" width="160"><input name="SUBNET_INPUT_1" maxlength="3" size="3" value="' .. number_or_blank(sub_mask_1[edit_filter]) .. '" type="text">.<input name="SUBNET_INPUT_2" maxlength="3" size="3" value="' .. number_or_blank(sub_mask_2[edit_filter]) .. '" type="text">.<input name="SUBNET_INPUT_3" maxlength="3" size="3" value="' .. number_or_blank(sub_mask_4[edit_filter]) .. '" type="text">.<input name="SUBNET_INPUT_4" maxlength="3" size="3" value="' .. number_or_blank(sub_mask_4[edit_filter]) .. '" type="text"></td>\n')
    put('    </tr>\n')
    put('    <tr>\n')
    put('      <td align="left"><p  title = "' ..current_dictionary("range").. ': 0..255">' ..current_dictionary("prot. no.")..'</p></td>\n')
    put('      <td align="left" width="160"><input name="PROT_NO_INPUT" maxlength="3" size="3" value="' .. number_or_blank(prot_no[edit_filter]) .. '" type="text"></td> \n')
    put('    </tr>\n')
    put('    <tr>\n')
    put('      <td align="left"><p title = "' ..current_dictionary("range").. ': 0..65535" >' ..string.gsub(current_dictionary("dest.port range"),"!"," ").. '</p></td>\n')
    put('      <td align="left" width="160"><p><input name="DEST_PORT_FROM_INPUT" maxlength="5" size="5" value="' .. number_or_blank(dest_port_range_from[edit_filter]) .. '" type="text"> ' .. current_dictionary("to") .. ' <input name="DEST_PORT_TO_INPUT" maxlength="5" size="5" value="' .. number_or_blank(dest_port_range_to[edit_filter]) .. '" type="text"></p></td> \n')
    put('    </tr>\n')
    put('    <tr>\n')
    put('      <td align="left"><p  title = "' ..current_dictionary("range").. ': 0..65535">' ..string.gsub(current_dictionary("source port range"),"!"," ").. '</p></td>\n')
    put('      <td align="left" width="160"><p><input name="SOURCE_PORT_FROM_INPUT" maxlength="5" size="5" value="' .. number_or_blank(source_port_range_from[edit_filter]) .. '" type="text"> ' .. current_dictionary("to") ..  ' <input name="SOURCE_PORT_TO_INPUT" maxlength="5" size="5" value="' .. number_or_blank(source_port_range_to[edit_filter]) .. '" type="text"></p></td>\n')
    put('    </tr>\n')
    put('    <tr>\n')
    put('      <td align="left"><p  title = "' ..current_dictionary("range").. ': 0..255" >' ..string.gsub(current_dictionary("type of service"),"!"," ").. '</p></td>\n')
    put('      <td align="left" width="160"><input name="SERVICE_TYPE" maxlength="3" size="3" value="' .. number_or_blank(type_of_service[edit_filter]) .. '" type="text"></td>\n')
    put('    </tr>\n')
    put('    <tr>\n')
    put('      <td align="left"><p  title = "' ..current_dictionary("range").. ': 0..255" >' ..string.gsub(current_dictionary("type of service mask"),"!"," ").. '</p></td>\n')
    put('      <td align="left" width="160"><input name="SERVICE_TYPE_MASK" maxlength="3" size="3" value="' .. number_or_blank(type_of_service_mask[edit_filter]) .. '" type="text"></td>\n')
    put('   </tr>\n')
    put('    <tr><td align="left" colspan="5"><input type="submit" name="submit_new" value="' ..current_dictionary("apply") .. '">&nbsp;<input type="submit"  name="submit_back" value="' ..current_dictionary("cancel") .. '"></TD></tr>\n')
    put(' </form> \n')
    put(' </tbody>\n')
    put('</table>\n')
else
    if error_val ~= 0 then
        put('<table align=center><tr><td><p id="errortext">\n')
        put(current_dictionary("error in filter")..'</p></td></tr></table>\n')
    end    

    put('<p id="traffic_flow_filters"><table width="95%" border="'..border_on..'" cellpadding="3">\n')
    put('<form method="post" action="index.lua?pageID=' .. page_title .. '&amp;langID=' .. lang .. ' ">\n')
    put('<tr>\n')
        put('<th align="center" title = "' ..string.gsub(current_dictionary("each value only once"),"NN1","0..19").. '"><p>' ..string.gsub(current_dictionary("eval.prec.index"),"!","<br>").. '</p></th>\n') 

        put('<th align="center" ><p>' .. current_dictionary("profile") .. '</p></th>\n') 
        put('<th align="center" title = "' ..current_dictionary("range for each dot-separated field:").. '0..255"><p>' ..current_dictionary("source address").. '</p></th>\n')
        put('<th align="center" title = "' ..current_dictionary("range for each dot-separated field:").. '0..255"><p>' ..current_dictionary("subnet mask").. '</p></th>\n')
        put('<th align="center" title = "' ..current_dictionary("range").. ': 0..255"><p>' ..current_dictionary("prot. no.").. '</p></th>\n')
        put('<th align="center" title = "' ..current_dictionary("range").. ': 0..65535"><p>' ..string.gsub(current_dictionary("dest.port range"),"!","<br>").. '</p></th>\n')  
        --put('<th align="center" width="7%"  title = "' ..current_dictionary("range: dest._port_range_from").. '..65535"><p>' ..string.gsub(current_dictionary("dest.port range"),"!","<br>") .. '<br>' ..current_dictionary("tft_to").. '</p></th>\n')
        put('<th align="center" title = "' ..current_dictionary("range").. ': 0..65535"><p>' ..string.gsub(current_dictionary("source port range"),"!","<br>").. '</p></th>\n')
        --put('<th align="center" width="7%"  title = "' ..current_dictionary("range: source_port_range_from").. '..65535"><p>' ..string.gsub(current_dictionary("source port range"),"!","<br>").. '<br>' ..current_dictionary("tft_to").. '</p></th>\n')
        put('<th align="center" title = "' ..current_dictionary("range").. ': 0..255"><p>' ..string.gsub(current_dictionary("type of service"),"!","<br>").. '</p></th>\n')
        put('<th align="center" title = "' ..current_dictionary("range").. ': 0..255"><p>' ..string.gsub(current_dictionary("type of service mask"),"!","<br>").. '</p></th>\n')
    put('</tr>\n')

    sort_by_epi()

    for i = 0, max_filter_id do
       if sorted_epi_list[i] ~= -1 then
           put_filter(sorted_epi_list[i])
        end
    end

    put('<tr></tr>\n')
    put('<tr>\n')
    if first_free_filter_index ~= -1 then
        put('<td align="left" colspan="4"><p><a href="index.lua?pageID=' .. page_title .. '&amp;langID=' .. lang .. '&amp;new_filter=1&amp;edit_filter=' .. first_free_filter_index .. '">' ..current_dictionary("new entry") .. '</a></p></td>\n\n')
    end 
    put('</tr></form>\n')
    put('</table></p>\n')
end

put(end_of_content)
